/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2008 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package pendragon.ui.app;

import static pendragon.i18n.I18n._;

import java.awt.SplashScreen;
import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import pendragon.Config;
import pendragon.ui.CoolText;
import pendragon.ui.app.actions.DescVisibleAction;
import pendragon.ui.app.actions.MainVisibleAction;
import pendragon.ui.app.actions.OpenConfigureWindowAction;
import pendragon.ui.app.actions.TagVisibleAction;
import pendragon.ui.common.actions.OpenWindowAction;
import pendragon.ui.config.ConfigWindow;
import pendragon.ui.mainview.MainView;
import pendragon.ui.tagview.TagView;

public class AppWindow extends ApplicationWindow {
    private static final String SIZE_SFORM2  = "ui.forms.sform2";
    private static final String SIZE_SFORM1  = "ui.forms.sform1";
    private static final String HEIGHT       = "ui.size[@height]";
    private static final String WIDTH        = "ui.size[@width]";
    private static final String MAXIMIZED    = "ui.size[@maximized]";
    private static final String TAG_VISIBLE  = "ui.view.tag[@visible]";
    private static final String MAIN_VISIBLE = "ui.view.main[@visible]";
    private static final String DESC_VISIBLE = "ui.view.desc[@visible]";
    private SashForm sform1;
    private SashForm sform2;
    private TagView tagView;
    private MainView mainView;
    private CoolText descText;
    public AppWindow() {
	super((Shell)null);
	setBlockOnOpen(true);
	addMenuBar();
    }
    public SashForm getSform1() {
        return sform1;
    }
    public SashForm getSform2() {
        return sform2;
    }
    public TagView getTagView() {
        return tagView;
    }
    public MainView getMainView() {
        return mainView;
    }
    public CoolText getDescText() {
        return descText;
    }
    public boolean getTagVisible() {
	return Config.xmlConfig.getBoolean(TAG_VISIBLE, true);
    }
    public boolean getMainVisible() {
	return Config.xmlConfig.getBoolean(MAIN_VISIBLE, true);
    }
    public boolean getDescVisible() {
	return Config.xmlConfig.getBoolean(DESC_VISIBLE, true);
    }
    @Override protected Control createContents(Composite parent) {
	Shell shell = getShell();
	shell.setText("Pendragon");
	sform1 = new SashForm(shell, SWT.HORIZONTAL);
	tagView = new TagView(sform1, SWT.NONE);
	sform2 = new SashForm(sform1, SWT.VERTICAL);
	mainView = new MainView(sform2, SWT.NONE);
	descText = new CoolText(sform2, SWT.BORDER | SWT.WRAP | SWT.READ_ONLY);
	readConfig();
	return parent;
    }
    @Override public int open() {
	Display.getDefault().timerExec(1000, new Runnable() {
	    @Override public void run() {
		SplashScreen splash = SplashScreen.getSplashScreen();
		if (splash != null) splash.close();
	    }
	});
	return super.open();
    }
    @Override protected MenuManager createMenuManager() {
	MenuManager menuBar = new MenuManager();
	//
	MenuManager fileBar = new MenuManager(_("File"));
	menuBar.add(fileBar);
	Action configWindow = new OpenConfigureWindowAction(_("Configuration"), new ConfigWindow(true));
	fileBar.add(configWindow);
	//
	MenuManager viewBar = new MenuManager(_("View"));
	menuBar.add(viewBar);
	TagVisibleAction tagAction = new TagVisibleAction(this);
	viewBar.add(tagAction);
	MainVisibleAction mainAction = new MainVisibleAction(this);
	viewBar.add(mainAction);
	DescVisibleAction descAction = new DescVisibleAction(this);
	viewBar.add(descAction);
	//
	MenuManager menuHelp = new MenuManager(_("Help"));
	menuBar.add(menuHelp);
	Action aboutDialog = new OpenWindowAction(_("About program"), new AboutDialog());
	menuHelp.add(aboutDialog);
	//
	return menuBar;
    }
    @SuppressWarnings("unchecked")
    private void readConfig() {
	Shell shell = getShell();
	Point preferedSize = shell.computeSize(640, 480);
	shell.setSize(Config.xmlConfig.getInt(WIDTH, preferedSize.x), Config.xmlConfig.getInt(HEIGHT, preferedSize.y));
	shell.setMaximized(Config.xmlConfig.getBoolean(MAXIMIZED, false));
	tagView.setVisible(getTagVisible());
	mainView.setVisible(getMainVisible());
	descText.setVisible(getDescVisible());
	List<String> weights = (List<String>)Config.xmlConfig.getList(SIZE_SFORM1, Arrays.asList("25", "75")); 
	sform1.setWeights(new int[] { Integer.valueOf(weights.get(0)), Integer.valueOf(weights.get(1)) });
	weights = (List<String>)Config.xmlConfig.getList(SIZE_SFORM2, Arrays.asList("75", "25")); 
	sform2.setWeights(new int[] { Integer.valueOf(weights.get(0)), Integer.valueOf(weights.get(1)) });
    }    
    @Override public boolean close() {
	Config.xmlConfig.setProperty(SIZE_SFORM1, Arrays.asList(sform1.getWeights()));
	Config.xmlConfig.setProperty(SIZE_SFORM2, Arrays.asList(sform2.getWeights()));
	Config.xmlConfig.setProperty(TAG_VISIBLE, tagView.getVisible());
	Config.xmlConfig.setProperty(MAIN_VISIBLE, mainView.getVisible());
	Config.xmlConfig.setProperty(DESC_VISIBLE, descText.getVisible());
	Shell shell = getShell();
	boolean maximized = shell.getMaximized();
	Config.xmlConfig.setProperty(MAXIMIZED, maximized);
	if (!maximized) {
	    Config.xmlConfig.setProperty(WIDTH, shell.getSize().x);
	    Config.xmlConfig.setProperty(HEIGHT, shell.getSize().y);		
	}
	return super.close();
    }
}
